<?php
/**
 * Очистка логов iskwaf2x для MODX2
 */


$rootPath = dirname(__FILE__, 6);
if (!file_exists($rootPath . '/config.core.php')) {
    die('Could not find MODX config.core.php. Adjust the path if necessary. Current base path assumed: ' . $rootPath);
}

require_once $rootPath . '/config.core.php';
require_once MODX_CORE_PATH . 'model/modx/modx.class.php';


$modx = new modX();
$modx->initialize('web'); 


if (php_sapi_name() !== 'cli' && (!defined('MODX_API_MODE') || MODX_API_MODE !== true)) {
    die('Access denied. This script is intended for CLI execution or MODX API mode.');
}


$corePath = $modx->getOption('core_path', null, MODX_CORE_PATH);
$modelPath = $corePath . 'components/iskwaf2x/model/';
if (!$modx->addPackage('iskwaf2x', $modelPath)) {
    if (php_sapi_name() === 'cli') {
        fwrite(STDERR, "[iskwaf2x Cleanup] Could not load iskwaf2x model package from: " . $modelPath . "\n");
    }
    die();
}

$logRetentionDaysSettingKey = 'iskwaf2x_iskwaf2x_log_retention_days';
$defaultRetentionDays = 7;
$logRetentionDays = (int) $modx->getOption($logRetentionDaysSettingKey, null, $defaultRetentionDays);

if ($logRetentionDays <= 0) {
    if (php_sapi_name() === 'cli') {
        echo "Warning: Invalid or missing '{$logRetentionDaysSettingKey}' setting (value: {$logRetentionDays}). Using default of {$defaultRetentionDays} days.\n";
    }
    $logRetentionDays = $defaultRetentionDays;
}

$startTime = microtime(true);
if (php_sapi_name() === 'cli') {
    echo "Starting log cleanup for entries older than " . $logRetentionDays . " days...\n";
}


$cutoffDate = date('Y-m-d H:i:s', strtotime('-' . $logRetentionDays . ' days'));

if (php_sapi_name() === 'cli') {
    echo "Cutoff date: " . $cutoffDate . "\n";
}


$criteriaArray = [
    'timestamp:<' => $cutoffDate,
];


$removedCount = $modx->removeCollection('IskWafLog', $criteriaArray);

if ($removedCount !== false) {
    if (php_sapi_name() === 'cli') {
        echo "Successfully removed " . $removedCount . " old log entries.\n";
    }
} else {
    if (php_sapi_name() === 'cli') {
        fwrite(STDERR, "Error removing old log entries. Check MODX error log for more details.\n");
    }
}

$endTime = microtime(true);
$executionTime = round($endTime - $startTime, 4);

if (php_sapi_name() === 'cli') {
    echo "Log cleanup finished in " . $executionTime . " seconds.\n";
}

exit(0);
