<?php
/**
 * Создание отчетов для iskwaf2x (MODX2)
 */

if (php_sapi_name() !== 'cli') {
    die('Access denied');
}

$rootPath = dirname(__FILE__, 6);
if (!file_exists($rootPath . '/config.core.php')) {
    die('Could not find MODX config.core.php. Adjust the path if necessary. Current base path assumed: ' . $rootPath);
}

require_once $rootPath . '/config.core.php';
require_once MODX_CORE_PATH . 'model/modx/modx.class.php';

$modx = new modX();
$modx->initialize('web'); 

$corePath = $modx->getOption('core_path', null, MODX_CORE_PATH);
$modelPath = $corePath . 'components/iskwaf2x/model/';
if (!$modx->addPackage('iskwaf2x', $modelPath)) {
    die('[iskwaf2x Reports] Could not load iskwaf2x model package from: ' . $modelPath);
}


$reportDate = date('Y-m-d', strtotime('-1 day'));


$existingReport = $modx->getObject('IskWafReportDaily', ['report_date' => $reportDate]);
if ($existingReport) {
    if (!$existingReport->remove()) {
        
    }
}


$reportData = [
    'report_date' => $reportDate,
    'total_triggered_events' => 0,
    'total_action_block' => 0,
    'total_action_captcha' => 0,
    'total_action_allow' => 0,
    'ip_rules_triggered' => 0,
    'ua_rules_triggered' => 0,
    'referrer_rules_triggered' => 0,
    'uri_rules_triggered' => 0,
    'hits_googlebot' => 0,
    'hits_bingbot' => 0,
    'hits_yandexbot' => 0,
    'hits_duckduckbot' => 0,
    'hits_baiduspider' => 0,
    'hits_applebot' => 0,
    'summary_generated_at' => date('Y-m-d H:i:s')
];

$topBlockedIps = [];
$topTriggeringRules = [];


$startDate = $reportDate . ' 00:00:00';
$endDate = date('Y-m-d H:i:s', strtotime($reportDate . ' +1 day') -1);

$c = $modx->newQuery('IskWafLog');
$c->where([
    'timestamp:>=' => $startDate,
    'timestamp:<=' => $endDate,
]);

$logs = $modx->getIterator('IskWafLog', $c);

foreach ($logs as $log) {
    $action = $log->get('waf_action');
    $triggeredRule = $log->get('triggered_rule');
    $userAgent = $log->get('user_agent');
    $ipAddress = $log->get('ip_address');

    if (!empty($triggeredRule) && $action !== 'VISIT') {
        $reportData['total_triggered_events']++;

        if ($action === 'block') {
            $reportData['total_action_block']++;
            if (!isset($topBlockedIps[$ipAddress])) {
                $topBlockedIps[$ipAddress] = 0;
            }
            $topBlockedIps[$ipAddress]++;
        } elseif ($action === 'captcha') {
            $reportData['total_action_captcha']++;
        } elseif ($action === 'allow') {
            $reportData['total_action_allow']++;
        }

        if (strpos($triggeredRule, 'ip:') === 0) {
            $reportData['ip_rules_triggered']++;
        } elseif (strpos($triggeredRule, 'ua:') === 0) {
            $reportData['ua_rules_triggered']++;
        } elseif (strpos($triggeredRule, 'ref:') === 0) {
            $reportData['referrer_rules_triggered']++;
        } elseif (strpos($triggeredRule, 'uri:') === 0) {
            $reportData['uri_rules_triggered']++;
        }

        if (!isset($topTriggeringRules[$triggeredRule])) {
            $topTriggeringRules[$triggeredRule] = 0;
        }
        $topTriggeringRules[$triggeredRule]++;
    }

    if (stripos($userAgent, 'Googlebot') !== false) $reportData['hits_googlebot']++;
    elseif (stripos($userAgent, 'bingbot') !== false || stripos($userAgent, 'MSNBot') !== false) $reportData['hits_bingbot']++;
    elseif (stripos($userAgent, 'YandexBot') !== false) $reportData['hits_yandexbot']++;
    elseif (stripos($userAgent, 'DuckDuckBot') !== false) $reportData['hits_duckduckbot']++;
    elseif (stripos($userAgent, 'Baiduspider') !== false) $reportData['hits_baiduspider']++;
    elseif (stripos($userAgent, 'Applebot') !== false) $reportData['hits_applebot']++;
}


arsort($topBlockedIps);
arsort($topTriggeringRules);
$limitTopN = 10;

$reportData['top_blocked_ips_json'] = json_encode(
    array_map(function($ip, $count) {
        return ['ip' => $ip, 'count' => $count];
    }, array_keys(array_slice($topBlockedIps, 0, $limitTopN, true)), array_slice($topBlockedIps, 0, $limitTopN, true))
);

$reportData['top_triggering_rules_json'] = json_encode(
    array_map(function($rule, $count) {
        return ['rule' => $rule, 'count' => $count];
    }, array_keys(array_slice($topTriggeringRules, 0, $limitTopN, true)), array_slice($topTriggeringRules, 0, $limitTopN, true))
);


$reportObject = $modx->newObject('IskWafReportDaily');
$reportObject->fromArray($reportData);

$reportObject->save(); 

exit(0);
