<?php
/**
 * IskWaf2x Guard File
 *
 * Этот файл-загрузчик инициализирует MODX 2 и запускает сервис безопасности IskWaf2x.
 * Его следует подключать в любые PHP-файлы, требующие защиты.
 */

// 1. Подключаем и инициализируем MODX
@include_once dirname(__FILE__, 4) . '/config.core.php';

if (!defined('MODX_CORE_PATH')) {
    // Безопасный выход, если не удалось найти конфигурацию MODX
    header('HTTP/1.1 503 Service Unavailable');
    die('Could not load MODX configuration.');
}
require_once MODX_CORE_PATH . 'model/modx/modx.class.php';

$modx = new modX();
$modx->initialize('web');


// 2. Подключаем и запускаем сервис безопасности iskwaf2x
$servicePath = $modx->getOption('core_path') . 'components/iskwaf2x/model/iskwaf2xservice.class.php';

if (!file_exists($servicePath)) {
    $modx->log(modX::LOG_LEVEL_ERROR, '[iskwaf2x] Critical: Service class file not found at: ' . $servicePath);
    header('HTTP/1.1 503 Service Unavailable');
    die('Security service is misconfigured.');
}
require_once $servicePath;

try {
    // Создаем экземпляр сервиса и запускаем его
    $waf = new IskWaf2xService($modx);
    $waf->run();

} catch (Exception $e) {
    $modx->log(modX::LOG_LEVEL_ERROR, '[iskwaf2x] Service execution failed in guard file: ' . $e->getMessage());
    header('HTTP/1.1 503 Service Unavailable');
    die('An error occurred in the security service.');
}

// Защита отработала. Управление возвращается к скрипту, который подключил этот файл.