<?php
/**
 * Получение списка логов + частичных ip-деталей для iskwaf2x (MODX2)
 */

class IskWaf2xItemGetListProcessor extends modObjectGetListProcessor
{
    public $objectType = 'IskWafLog';
    public $classKey = 'IskWafLog';
    public $defaultSortField = 'id';
    public $defaultSortDirection = 'DESC';
    public $languageTopics = ['iskwaf2x:default'];
    

    public function beforeQuery()
    {
        if (!$this->checkPermissions()) {
            return $this->modx->lexicon('access_denied');
        }
        return true;
    }

    public function prepareQueryBeforeCount(xPDOQuery $c)
    {
        
        $c->leftJoin('IskWafIp2LocationDetail', 'IpDetail', 'IskWafLog.ip_address = IpDetail.ip_address');

        
        $c->select($this->modx->getSelectColumns('IskWafLog', 'IskWafLog'));

        
        $c->select($this->modx->getSelectColumns('IskWafIp2LocationDetail', 'IpDetail', 'ipdetail_', ['country_name', 'city_name', 'as_name','asn']));

        $query = trim($this->getProperty('query'));
        if (!empty($query)) {
            $conditions = array();
            $isNumericQuery = is_numeric($query);
            $numericValue = $isNumericQuery ? intval($query) : null;

            
            $conditions['IskWafLog.ip_address:LIKE'] = "%{$query}%";
            $conditions['OR:IskWafLog.method:LIKE'] = "%{$query}%";
            $conditions['OR:IskWafLog.request_uri:LIKE'] = "%{$query}%";
            $conditions['OR:IskWafLog.user_agent:LIKE'] = "%{$query}%";
            $conditions['OR:IskWafLog.referrer:LIKE'] = "%{$query}%";
            $conditions['OR:IskWafLog.waf_action:LIKE'] = "%{$query}%";
            $conditions['OR:IskWafLog.triggered_rule:LIKE'] = "%{$query}%";
            $conditions['OR:IskWafLog.notes:LIKE'] = "%{$query}%";
            $conditions['OR:IskWafLog.timestamp:LIKE'] = "%{$query}%";

            if ($isNumericQuery) {
                $conditions['OR:IskWafLog.id:='] = $numericValue;
                $conditions['OR:IskWafLog.user_id:='] = $numericValue;
                $conditions['OR:IskWafLog.http_status:='] = $numericValue;
            }

            
            $conditions['OR:IpDetail.country_name:LIKE'] = "%{$query}%";
            $conditions['OR:IpDetail.city_name:LIKE'] = "%{$query}%";
            $conditions['OR:IpDetail.as_name:LIKE'] = "%{$query}%";
            $conditions['OR:IpDetail.asn:LIKE'] = "%{$query}%";

            if (!empty($conditions)) {
                $c->where($conditions);
            }
        }
        return $c;
    }

    public function prepareRow(xPDOObject $object)
    {
        $array = $object->toArray();

        
        $array['actions'] = array();
        $array['actions'][] = array(
            'cls' => '',
            'icon' => 'icon icon-edit',
            'title' => $this->modx->lexicon('iskwaf2x_item_update'),
            'action' => 'updateItem',
            'button' => true,
            'menu' => true,
        );
        $array['actions'][] = array(
            'cls' => '',
            'icon' => 'icon icon-trash-o action-red',
            'title' => $this->modx->lexicon('iskwaf2x_item_remove'),
            'multiple' => $this->modx->lexicon('iskwaf2x_items_remove'),
            'action' => 'removeItem',
            'button' => true,
            'menu' => true,
        );

        return $array;
    }
}

return 'IskWaf2xItemGetListProcessor';
