<?php
/**
 * Создание правила IskWafRuleIp для iskwaf2x (MODX2)
 */

class IskWaf2xItem2CreateProcessor extends modObjectCreateProcessor
{
    public $objectType = 'IskWafRuleIp';
    public $classKey = 'IskWafRuleIp';
    public $languageTopics = array('iskwaf2x:default');
    

    /**
     * @return bool
     */
    public function beforeSet()
    {
        $ipCidr = trim($this->getProperty('ip_cidr'));
        $isValidFormat = false; 

        if (empty($ipCidr)) {
            $this->modx->error->addField('ip_cidr', $this->modx->lexicon('iskwaf2x_iprule_err_ip_cidr_ns'));
        } else {
            
            if (strpos($ipCidr, '/') !== false) {
                $parts = explode('/', $ipCidr, 2);
                if (count($parts) === 2) {
                    $ip = $parts[0];
                    $mask = $parts[1];
                    if (filter_var($ip, FILTER_VALIDATE_IP) && ctype_digit($mask)) {
                        $maskInt = (int)$mask;
                        if (filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4)) {
                            if ($maskInt >= 0 && $maskInt <= 32) {
                                $isValidFormat = true;
                            }
                        } elseif (filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV6)) {
                            if ($maskInt >= 0 && $maskInt <= 128) {
                                $isValidFormat = true;
                            }
                        }
                    }
                }
            } else {
                if (filter_var($ipCidr, FILTER_VALIDATE_IP)) {
                    $isValidFormat = true;
                }
            }

            if (!$isValidFormat) {
                $this->modx->error->addField('ip_cidr', $this->modx->lexicon('iskwaf2x_iprule_err_ip_cidr_invalid'));
            } else {
                
                if ($this->modx->getCount($this->classKey, array('ip_cidr' => $ipCidr))) {
                    $this->modx->error->addField('ip_cidr', $this->modx->lexicon('iskwaf2x_iprule_err_ip_cidr_ae'));
                }
            }
        }
        return parent::beforeSet();
    }
}

return 'IskWaf2xItem2CreateProcessor';
