iskwaf2x.grid.Items = function (config) {
    config = config || {};
    if (!config.id) {
        config.id = 'iskwaf2x-grid-items';
    }
    Ext.applyIf(config, {
        url: iskwaf2x.config.connector_url,
        fields: this.getFields(config),
        columns: this.getColumns(config),
        tbar: this.getTopBar(config),
        sm: new Ext.grid.CheckboxSelectionModel(),
        baseParams: {
            action: 'mgr/item/getlist',
        },
        listeners: {
            rowDblClick: function (grid, rowIndex, e) {
                const row = grid.store.getAt(rowIndex);
                this.updateItem(grid, e, row);
            }
        },
        viewConfig: {
            forceFit: true,
            enableRowBody: true,
            autoFill: true,
            showPreview: true,
            scrollOffset: 0,
            getRowClass: function (rec) {
                return !rec.data.active
                    ? 'iskwaf2x-grid-row-disabled'
                    : '';
            }
        },
        paging: true,
        remoteSort: true,
        autoHeight: true,
    });
    iskwaf2x.grid.Items.superclass.constructor.call(this, config);

    this.store.on('load', function () {
        if (this._getSelectedIds().length) { this.getSelectionModel().clearSelections(); }
    }, this);
};
Ext.extend(iskwaf2x.grid.Items, MODx.grid.Grid, {
    windows: {},

    getMenu: function (grid, rowIndex) {
        const ids = this._getSelectedIds();
        const row = grid.getStore().getAt(rowIndex);
        let actions = row.data['actions'];

        if (actions && Array.isArray(actions)) {
            const actionsToExclude = [
                'enable',
                'disable',
                'enableitem',
                'disableitem',
                'publish',
                'unpublish',
                'activate',
                'deactivate'
            ];

            actions = actions.filter(function(actionItem) {
                let actionKey = '';
                if (typeof actionItem === 'string') {
                    actionKey = actionItem.toLowerCase();
                } else if (actionItem && typeof actionItem.action === 'string') {
                    actionKey = actionItem.action.toLowerCase();
                } else if (actionItem && typeof actionItem.handler === 'string') {
                    actionKey = actionItem.handler.toLowerCase();
                }
                return !actionsToExclude.includes(actionKey);
            });
        }

        const menu = iskwaf2x.utils.getMenu(actions, this, ids);
        this.addContextMenuItem(menu);
    },

    updateItem: function (btn, e, row) {
        if (typeof(row) != 'undefined') { this.menu.record = row.data; } else if (!this.menu.record) { return false; } const id = this.menu.record.id;
        MODx.Ajax.request({ url: this.config.url, params: { action: 'mgr/item/get', id: id }, listeners: { success: { fn: function (r) { const w = MODx.load({ xtype: 'iskwaf2x-Item-window-update', id: Ext.id(), record: r, listeners: { success: { fn: function () { this.refresh(); }, scope: this } } }); w.reset(); w.setValues(r.object); w.show(e.target); }, scope: this } } });
    },
    removeItem: function () {
        const ids = this._getSelectedIds(); if (!ids.length) { return false; }
        MODx.msg.confirm({ title: ids.length > 1 ? _('iskwaf2x_items_remove') : _('iskwaf2x_item_remove'), text: ids.length > 1 ? _('iskwaf2x_items_remove_confirm') : _('iskwaf2x_item_remove_confirm'), url: this.config.url, params: { action: 'mgr/item/remove', ids: Ext.util.JSON.encode(ids) }, listeners: { success: { fn: function () { this.refresh(); }, scope: this } } }); return true;
    },

    getFields: function () {
        return [
            'id', 'timestamp', 'ip_address',
            'ipdetail_country_name',
            'ipdetail_city_name',

            'method', 'request_uri', 'user_agent', 'referrer', 'http_status',
            'waf_action', 'triggered_rule', 'notes',
            'actions'
        ];
    },

    getColumns: function () {
        const renderEllipsis = function(value, maxLength) {
            value = value || '';
            if (value.length > maxLength) {
                return Ext.util.Format.htmlEncode(value.substring(0, maxLength - 3)) + '...';
            }
            return Ext.util.Format.htmlEncode(value);
        };

        return [{
            header: _('id'),
            dataIndex: 'id',
            sortable: true,
            width: 10,
            hidden: true
        }, {
            header: _('iskwaf2x_log_timestamp'),
            dataIndex: 'timestamp',
            sortable: true,
            width: 140
        }, {
            header: _('iskwaf2x_log_ip_address'),
            dataIndex: 'ip_address',
            sortable: true,
            width: 110
        }, {
            header: _('iskwaf2x_log_country'),
            dataIndex: 'ipdetail_country_name',
            sortable: true,
            width: 110
        }, {
            header: _('iskwaf2x_log_city'),
            dataIndex: 'ipdetail_city_name',
            sortable: true,
            width: 110
        }, {
            header: _('iskwaf2x_log_method'),
            dataIndex: 'method',
            sortable: true,
            width: 60
        }, {
            header: _('iskwaf2x_log_request_uri'),
            dataIndex: 'request_uri',
            sortable: true,
            width: 150,
            renderer: function(value) { return renderEllipsis(value, 35); }
        }, {
            header: _('iskwaf2x_log_referrer'),
            dataIndex: 'referrer',
            sortable: true,
            width: 150,
            renderer: function(value) { return renderEllipsis(value, 35); }
        }, {
            header: _('iskwaf2x_log_user_agent'),
            dataIndex: 'user_agent',
            sortable: true,
            width: 180,
            renderer: function(value) { return renderEllipsis(value, 40); }
        }, {
            header: _('iskwaf2x_log_http_status'),
            dataIndex: 'http_status',
            sortable: true,
            width: 70
        }, {
            header: _('iskwaf2x_log_waf_action'),
            dataIndex: 'waf_action',
            sortable: true,
            width: 90
        }, {
            header: _('iskwaf2x_log_triggered_rule'),
            dataIndex: 'triggered_rule',
            sortable: true,
            width: 110
        },{
            header: _('iskwaf2x_log_notes'),
            dataIndex: 'notes',
            sortable: false,
            width: 130,
            renderer: function(value) { return renderEllipsis(value, 25); }
        }];
    },

    getTopBar: function () {
        return ['->', {
            xtype: 'iskwaf2x-field-search',
            width: 250,
            listeners: {
                search: { fn: function (field) { this._doSearch(field); }, scope: this },
                clear: { fn: function (field) { field.setValue(''); this._clearSearch(); }, scope: this },
            }
        }];
    },
    onClick: function (e) {
        const elem = e.getTarget(); if (elem.nodeName == 'BUTTON') { const row = this.getSelectionModel().getSelected(); if (typeof(row) != 'undefined') { const action = elem.getAttribute('action'); if (action == 'showMenu') { const ri = this.getStore().find('id', row.id); return this._showMenu(this, ri, e); } else if (typeof this[action] === 'function') { this.menu.record = row.data; return this[action](this, e); } } } return this.processEvent('click', e);
    },
    _getSelectedIds: function () {
        const ids = []; const selected = this.getSelectionModel().getSelections(); for (const i in selected) { if (!selected.hasOwnProperty(i)) { continue; } ids.push(selected[i]['id']); } return ids;
    },
    _doSearch: function (tf) {
        this.getStore().baseParams.query = tf.getValue(); this.getBottomToolbar().changePage(1);
    },
    _clearSearch: function () {
        this.getStore().baseParams.query = ''; this.getBottomToolbar().changePage(1);
    },
});
Ext.reg('iskwaf2x-grid-items', iskwaf2x.grid.Items);