iskwaf2x.grid.Items3 = function (config) {
    config = config || {};
    if (!config.id) {
        config.id = 'iskwaf2x-grid-items3';
    }
    Ext.applyIf(config, {
        url: iskwaf2x.config.connector_url,
        fields: this.getFields(config),
        columns: this.getColumns(config),
        tbar: this.getTopBar(config),
        sm: new Ext.grid.CheckboxSelectionModel(),
        baseParams: {
            action: 'mgr/item3/getlist',
        },
        listeners: {
            rowDblClick: function (grid, rowIndex, e) {
                const row = grid.store.getAt(rowIndex);
                this.updateItem(grid, e, row);
            }
        },
        viewConfig: {
            forceFit: true,
            enableRowBody: true,
            autoFill: true,
            showPreview: true,
            scrollOffset: 0,
            getRowClass: function (rec) {
                return !rec.data.active
                    ? 'iskwaf2x-grid-row-disabled'
                    : '';
            }
        },
        paging: true,
        remoteSort: true,
        autoHeight: true,
    });
    iskwaf2x.grid.Items3.superclass.constructor.call(this, config);

    this.store.on('load', function () {
        if (this._getSelectedIds().length) {
            this.getSelectionModel().clearSelections();
        }
    }, this);
};
Ext.extend(iskwaf2x.grid.Items3, MODx.grid.Grid, {
    windows: {},

    getMenu: function (grid, rowIndex) {
        const ids = this._getSelectedIds();
        const row = grid.getStore().getAt(rowIndex);
        const menu = iskwaf2x.utils.getMenu(row.data['actions'], this, ids);
        this.addContextMenuItem(menu);
    },

    createItem: function (btn, e) {
        const w = MODx.load({
            xtype: 'iskwaf2x-Item3-window-create',
            id: Ext.id(),
            listeners: {
                success: {
                    fn: function () {
                        this.refresh();
                    }, scope: this
                }
            }
        });
        w.reset();
        w.setValues({active: true});
        w.show(e.target);
    },

    updateItem: function (btn, e, row) {
        if (typeof(row) != 'undefined') {
            this.menu.record = row.data;
        }
        else if (!this.menu.record) {
            return false;
        }
        const id = this.menu.record.id;

        MODx.Ajax.request({
            url: this.config.url,
            params: {
                action: 'mgr/item3/get',
                id: id
            },
            listeners: {
                success: {
                    fn: function (r) {
                        const w = MODx.load({
                            xtype: 'iskwaf2x-Item3-window-update',
                            id: Ext.id(),
                            record: r,
                            listeners: {
                                success: {
                                    fn: function () {
                                        this.refresh();
                                    }, scope: this
                                }
                            }
                        });
                        w.reset();
                        w.setValues(r.object);
                        w.show(e.target);
                    }, scope: this
                }
            }
        });
    },

    removeItem: function () {
        const ids = this._getSelectedIds();
        if (!ids.length) {
            return false;
        }
        MODx.msg.confirm({
            title: ids.length > 1
                ? _('iskwaf2x_items_remove')
                : _('iskwaf2x_item_remove'),
            text: ids.length > 1
                ? _('iskwaf2x_items_remove_confirm')
                : _('iskwaf2x_item_remove_confirm'),
            url: this.config.url,
            params: {
                action: 'mgr/item3/remove',
                ids: Ext.util.JSON.encode(ids),
            },
            listeners: {
                success: {
                    fn: function () {
                        this.refresh();
                    }, scope: this
                }
            }
        });
        return true;
    },

    disableItem: function () {
        const ids = this._getSelectedIds();
        if (!ids.length) {
            return false;
        }
        MODx.Ajax.request({
            url: this.config.url,
            params: {
                action: 'mgr/item3/disable',

                ids: Ext.util.JSON.encode(ids),
            },
            listeners: {
                success: {
                    fn: function () {
                        this.refresh();
                    }, scope: this
                }
            }
        })
    },

    enableItem: function () {
        const ids = this._getSelectedIds();
        if (!ids.length) {
            return false;
        }
        MODx.Ajax.request({
            url: this.config.url,
            params: {
                action: 'mgr/item3/enable',
                ids: Ext.util.JSON.encode(ids),
            },
            listeners: {
                success: {
                    fn: function () {
                        this.refresh();
                    }, scope: this
                }
            }
        })
    },

    getFields: function () {
        return [
            'id',
            'user_agent_pattern',
            'pattern_type',
            'rule_type',
            'description',
            'active',
            'actions'
        ];
    },

    getColumns: function () {
        const renderEllipsis = function(value, maxLength) {
            value = value || '';
            if (value.length > maxLength) {
                return Ext.util.Format.htmlEncode(value.substring(0, maxLength - 3)) + '...';
            }
            return Ext.util.Format.htmlEncode(value);
        };

        return [{
            header: _('id'),
            dataIndex: 'id',
            sortable: true,
            width: 50,
            hidden: true
        }, {
            header: _('iskwaf2x_useragentrule_pattern'),
            dataIndex: 'user_agent_pattern',
            sortable: false,
            width: 300,
            renderer: function(value) { return renderEllipsis(value, 100); }
        }, {
            header: _('iskwaf2x_useragentrule_pattern_type'),
            dataIndex: 'pattern_type',
            sortable: true,
            width: 120
        }, {
            header: _('iskwaf2x_useragentrule_rule_type'),
            dataIndex: 'rule_type',
            sortable: true,
            width: 100
        }, {
            header: _('iskwaf2x_useragentrule_description'),
            dataIndex: 'description',
            sortable: false,
            width: 250,
            renderer: function(value) { return renderEllipsis(value, 70); }
        }, {
            header: _('iskwaf2x_useragentrule_active'),
            dataIndex: 'active',
            renderer: iskwaf2x.utils.renderBoolean,
            sortable: true,
            width: 70
        }, {
            header: _('iskwaf2x_grid_actions'),
            dataIndex: 'actions',
            renderer: iskwaf2x.utils.renderActions,
            sortable: false,
            width: 100,
            id: 'actions'
        }];
    },

    getTopBar: function () {
        return [{
            text: '<i class="icon icon-plus"></i>&nbsp;' + _('iskwaf2x_create_rule'),
            handler: this.createItem,
            scope: this
        }, '->', {
            xtype: 'iskwaf2x-field-search',
            width: 250,
            listeners: {
                search: {
                    fn: function (field) {
                        this._doSearch(field);
                    }, scope: this
                },
                clear: {
                    fn: function (field) {
                        field.setValue('');
                        this._clearSearch();
                    }, scope: this
                },
            }
        }];
    },

    onClick: function (e) {
        const elem = e.getTarget();
        if (elem.nodeName == 'BUTTON') {
            const row = this.getSelectionModel().getSelected();
            if (typeof(row) != 'undefined') {
                const action = elem.getAttribute('action');
                if (action == 'showMenu') {
                    const ri = this.getStore().find('id', row.id);
                    return this._showMenu(this, ri, e);
                }
                else if (typeof this[action] === 'function') {
                    this.menu.record = row.data;
                    return this[action](this, e);
                }
            }
        }
        return this.processEvent('click', e);
    },

    _getSelectedIds: function () {
        const ids = [];
        const selected = this.getSelectionModel().getSelections();

        for (const i in selected) {
            if (!selected.hasOwnProperty(i)) {
                continue;
            }
            ids.push(selected[i]['id']);
        }

        return ids;
    },

    _doSearch: function (tf) {
        this.getStore().baseParams.query = tf.getValue();
        this.getBottomToolbar().changePage(1);
    },

    _clearSearch: function () {
        this.getStore().baseParams.query = '';
        this.getBottomToolbar().changePage(1);
    },
});
Ext.reg('iskwaf2x-grid-items3', iskwaf2x.grid.Items3);