iskwaf2x.grid.Items6 = function (config) {
    config = config || {};
    if (!config.id) {
        config.id = 'iskwaf2x-grid-items6';
    }
    Ext.applyIf(config, {
        url: iskwaf2x.config.connector_url,
        fields: this.getFields(config),
        columns: this.getColumns(config),
        tbar: this.getTopBar(config),
        sm: new Ext.grid.CheckboxSelectionModel(),
        baseParams: {
            action: 'mgr/item6/getList',
        },
        listeners: {
            rowDblClick: function (grid, rowIndex, e) {
                const row = grid.store.getAt(rowIndex);
                this.updateItem(grid, e, row);
            }
        },
        viewConfig: {
            forceFit: true,
            enableRowBody: true,
            autoFill: true,
            showPreview: true,
            scrollOffset: 0,
            getRowClass: function (rec) {
                return !rec.data.active
                    ? 'iskwaf2x-grid-row-disabled'
                    : '';
            }
        },
        paging: true,
        remoteSort: true,
        autoHeight: true,
    });
    iskwaf2x.grid.Items6.superclass.constructor.call(this, config);

    // Clear selection on grid refresh
    this.store.on('load', function () {
        if (this._getSelectedIds().length) {
            this.getSelectionModel().clearSelections();
        }
    }, this);
};
Ext.extend(iskwaf2x.grid.Items6, MODx.grid.Grid, {
    windows: {},

    getMenu: function (grid, rowIndex) {
        const ids = this._getSelectedIds();

        const row = grid.getStore().getAt(rowIndex);
        const menu = iskwaf2x.utils.getMenu(row.data['actions'], this, ids);

        this.addContextMenuItem(menu);
    },

    createItem: function (btn, e) {
        const w = MODx.load({
            xtype: 'iskwaf2x-Item6-window-create',
            id: Ext.id(),
            listeners: {
                success: {
                    fn: function () {
                        this.refresh();
                    }, scope: this
                }
            }
        });
        w.reset();
        w.setValues({active: true});
        w.show(e.target);
    },

    updateItem: function (btn, e, row) {
        if (typeof(row) != 'undefined') {
            this.menu.record = row.data;
        }
        else if (!this.menu.record) {
            return false;
        }
        const id = this.menu.record.id;

        MODx.Ajax.request({
            url: this.config.url,
            params: {
                action: 'mgr/item6/get',
                id: id
            },
            listeners: {
                success: {
                    fn: function (r) {
                        const w = MODx.load({
                            xtype: 'iskwaf2x-Item6-window-update',
                            id: Ext.id(),
                            record: r,
                            listeners: {
                                success: {
                                    fn: function () {
                                        this.refresh();
                                    }, scope: this
                                }
                            }
                        });
                        w.reset();
                        w.setValues(r.object);
                        w.show(e.target);
                    }, scope: this
                }
            }
        });
    },

    removeItem: function () {
        const ids = this._getSelectedIds();
        if (!ids.length) {
            return false;
        }
        MODx.msg.confirm({
            title: ids.length > 1
                ? _('iskwaf2x_items_remove')
                : _('iskwaf2x_item_remove'),
            text: ids.length > 1
                ? _('iskwaf2x_items_remove_confirm')
                : _('iskwaf2x_item_remove_confirm'),
            url: this.config.url,
            params: {
                action: 'mgr/item6/remove',
                ids: Ext.util.JSON.encode(ids),
            },
            listeners: {
                success: {
                    fn: function () {
                        this.refresh();
                    }, scope: this
                }
            }
        });
        return true;
    },

    disableItem: function () {
        const ids = this._getSelectedIds();
        if (!ids.length) {
            return false;
        }
        MODx.Ajax.request({
            url: this.config.url,
            params: {
                action: 'mgr/item6/disable',
                ids: Ext.util.JSON.encode(ids),
            },
            listeners: {
                success: {
                    fn: function () {
                        this.refresh();
                    }, scope: this
                }
            }
        })
    },

    enableItem: function () {
        const ids = this._getSelectedIds();
        if (!ids.length) {
            return false;
        }
        MODx.Ajax.request({
            url: this.config.url,
            params: {
                action: 'mgr/item6/enable',
                ids: Ext.util.JSON.encode(ids),
            },
            listeners: {
                success: {
                    fn: function () {
                        this.refresh();
                    }, scope: this
                }
            }
        })
    },

    getFields: function () {
        return [
            'id',
            'ip_address',
            'country_code',
            'country_name',
            'region_name',
            'city_name',
            'latitude',
            'longitude',
            'zip_code',
            'time_zone',
            'asn',
            'as_name',
            'last_checked',
            'actions'
        ];
    },

    getColumns: function () {
        return [{
            header: 'ID',
            dataIndex: 'id',
            sortable: true,
            width: 20,
        }, {
            header: _('iskwaf2x_ip_address'),
            dataIndex: 'ip_address',
            sortable: true,
            width: 100,
        }, {
            header: _('iskwaf2x_country_code'),
            dataIndex: 'country_code',
            sortable: true,
            width: 40,
        }, {
            header: _('iskwaf2x_country_name'),
            dataIndex: 'country_name',
            sortable: true,
            width: 80,
        }, {
            header: _('iskwaf2x_region_name'),
            dataIndex: 'region_name',
            sortable: true,
            width: 80,
        }, {
            header: _('iskwaf2x_city_name'),
            dataIndex: 'city_name',
            sortable: true,
            width: 80,
        }, {
            header: _('iskwaf2x_latitude'),
            dataIndex: 'latitude',
            sortable: false,
            width: 50,
        }, {
            header: _('iskwaf2x_longitude'),
            dataIndex: 'longitude',
            sortable: false,
            width: 50,
        }, {
            header: _('iskwaf2x_zip_code'),
            dataIndex: 'zip_code',
            sortable: false,
            width: 60,
        }, {
            header: _('iskwaf2x_time_zone'),
            dataIndex: 'time_zone',
            sortable: false,
            width: 40,
        }, {
            header: _('iskwaf2x_asn'),
            dataIndex: 'asn',
            sortable: false,
            width: 60,
        }, {
            header: _('iskwaf2x_as_name'),
            dataIndex: 'as_name',
            sortable: false,
            width: 120,
        }, {
            header: _('iskwaf2x_last_checked'),
            dataIndex: 'last_checked',
            sortable: true,
            width: 90,
            renderer: Ext.util.Format.dateRenderer(MODx.config.manager_date_format + ' ' + MODx.config.manager_time_format)
        }, {
            header: _('iskwaf2x_grid_actions'),
            dataIndex: 'actions',
            renderer: iskwaf2x.utils.renderActions,
            sortable: false,
            width: 100,
            id: 'actions'
        }];
    },

    // --- остальное не тронуто ---

    getTopBar: function () {
        return [{
            text: '<i class="icon icon-trash"></i>&nbsp;' + _('iskwaf2x_clear_table'),
            handler: this.clearTable,
            scope: this
        }, '->', {
            xtype: 'iskwaf2x-field-search',
            width: 250,
            listeners: {
                search: {
                    fn: function (field) {
                        this._doSearch(field);
                    }, scope: this
                },
                clear: {
                    fn: function (field) {
                        field.setValue('');
                        this._clearSearch();
                    }, scope: this
                },
            }
        }];
    },

    clearTable: function () {
        MODx.msg.confirm({
            title: _('iskwaf2x_clear_table'),
            text: _('iskwaf2x_clear_table_confirm'),
            url: this.config.url,
            params: {
                action: 'mgr/item6/cleartable'
            },
            listeners: {
                success: {
                    fn: function () {
                        this.refresh();
                    }, scope: this
                }
            }
        });
    },


    onClick: function (e) {
        const elem = e.getTarget();
        if (elem.nodeName == 'BUTTON') {
            const row = this.getSelectionModel().getSelected();
            if (typeof(row) != 'undefined') {
                const action = elem.getAttribute('action');
                if (action == 'showMenu') {
                    const ri = this.getStore().find('id', row.id);
                    return this._showMenu(this, ri, e);
                }
                else if (typeof this[action] === 'function') {
                    this.menu.record = row.data;
                    return this[action](this, e);
                }
            }
        }
        return this.processEvent('click', e);
    },

    _getSelectedIds: function () {
        const ids = [];
        const selected = this.getSelectionModel().getSelections();

        for (const i in selected) {
            if (!selected.hasOwnProperty(i)) {
                continue;
            }
            ids.push(selected[i]['id']);
        }

        return ids;
    },

    _doSearch: function (tf) {
        this.getStore().baseParams.query = tf.getValue();
        this.getBottomToolbar().changePage(1);
    },

    _clearSearch: function () {
        this.getStore().baseParams.query = '';
        this.getBottomToolbar().changePage(1);
    },
});
Ext.reg('iskwaf2x-grid-items6', iskwaf2x.grid.Items6);
